"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CSSBuilder = void 0;
var DefaultConfig_1 = require("../common/DefaultConfig");
var cleanCSS = require('clean-css');
var fse = require('fs-extra');
var formatCSS = require('cssbeautify');
var sass = require('node-sass');
var postcss = require('postcss');
var autoprefixer = require('autoprefixer');
var BASE_SCSS_REGEX = new RegExp("(" + DefaultConfig_1.DefaultConfig.baseSCSSFiles.join('|') + ")");
var CSSBuilder = /** @class */ (function () {
    function CSSBuilder() {
    }
    /**
     * Generates minified and non-minified version of the CSS, based on the specified build data.
     */
    CSSBuilder.build = function (buildData) {
        // Compile the `core` module without the layout.
        // By default the `core` module includes the layout.
        var coreNoLayout = this._compileSCSS(this._loadStyles(buildData.files.scss
            .sort(function (path) { return BASE_SCSS_REGEX.test(path) ? -1 : 1; })
            .filter(function (path) { return path.indexOf('core') !== -1; })));
        // CSS for the components, without any layouts or structure.
        var componentCSS = this._loadStyles(buildData.files.css.filter(function (path) { return path.indexOf('core.css') === -1; }));
        return this._buildOutput(buildData, coreNoLayout + componentCSS, this._loadStyles(buildData.files.css));
    };
    /**
     * Generates a minified and non-minified version of the specified stylesheet.
     */
    CSSBuilder._buildStylesheet = function (styleSheet, license) {
        var compressed = new cleanCSS({
            // Strip the licensing info from the original file. It'll be re-added by the MaterialTools.
            keepSpecialComments: 0
        }).minify(styleSheet);
        return {
            source: this._beautifyStylesheet(styleSheet),
            compressed: compressed.styles,
            license: license
        };
    };
    /** Reads and concatenates CSS files */
    CSSBuilder._loadStyles = function (files) {
        return files.map(function (path) { return fse.readFileSync(path).toString(); }).join('\n');
    };
    /**
     * Compiles SCSS to CSS and passes it through Autoprefixer.
     */
    CSSBuilder._compileSCSS = function (styles) {
        var compiled = sass.renderSync({
            data: styles,
            outputStyle: 'compressed'
        }).css.toString();
        var prefixer = autoprefixer({
            overrideBrowserslist: ['last 2 versions', 'last 4 Android versions']
        });
        return postcss(prefixer).process(compiled).css;
    };
    /** Beautifies the specified CSS stylesheet */
    CSSBuilder._beautifyStylesheet = function (styleSheet) {
        return formatCSS(styleSheet, {
            indent: '  ',
            autosemicolon: true
        });
    };
    /** Builds an output object for the generated CSS from the build data. */
    CSSBuilder._buildOutput = function (data, noLayout, layout) {
        return {
            withoutLayout: this._buildStylesheet(noLayout, data.license),
            withLayout: this._buildStylesheet(layout, data.license),
            layoutFiles: data.files.layout,
            license: data.license
        };
    };
    return CSSBuilder;
}());
exports.CSSBuilder = CSSBuilder;
//# sourceMappingURL=CSSBuilder.js.map