"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MaterialBuilder = void 0;
var path = require("path");
var ThemeBuilder_1 = require("./ThemeBuilder");
var PackageResolver_1 = require("../resolvers/PackageResolver");
var DependencyResolver_1 = require("../resolvers/DependencyResolver");
var LocalResolver_1 = require("../resolvers/LocalResolver");
var JSBuilder_1 = require("./JSBuilder");
var CSSBuilder_1 = require("./CSSBuilder");
var DefaultConfig_1 = require("../common/DefaultConfig");
var MaterialBuilder = /** @class */ (function () {
    function MaterialBuilder(_options) {
        this._options = _options;
        this._outputBase = path.join(this._options.destination, this._options.destinationFilename);
    }
    /**
     * Figures out all the dependencies and necessary files for a build, based on the options.
     */
    MaterialBuilder.prototype._getData = function () {
        var _this = this;
        return PackageResolver_1.PackageResolver
            .resolve(this._options.version, this._options.cache)
            .then(function (versionData) {
            // Update the resolved version, in case it was `node`.
            _this._options.version = versionData.version;
            return {
                versionData: versionData,
                dependencies: DependencyResolver_1.DependencyResolver.resolve(_this._getModuleEntry(versionData), _this._options.modules)
            };
        })
            .then(function (data) {
            return LocalResolver_1.LocalResolver.resolve(data.dependencies._flat, data.versionData).then(function (files) {
                return {
                    files: files,
                    dependencies: data.dependencies,
                    license: _this._getLicense(data.dependencies._flat),
                    package: data.versionData
                };
            });
        });
    };
    /**
     * Outputs the necessary JS, based on the options.
     */
    MaterialBuilder.prototype._buildJS = function (buildData) {
        return JSBuilder_1.JSBuilder.build(buildData, this._outputBase + ".min.js");
    };
    /**
     * Outputs the CSS, based on the options.
     */
    MaterialBuilder.prototype._buildCSS = function (buildData) {
        return CSSBuilder_1.CSSBuilder.build(buildData);
    };
    /**
     * Outputs a static theme stylesheet, based on the specified options
     */
    MaterialBuilder.prototype._buildTheme = function (buildData) {
        if (!this._options.theme && !this._options.themes) {
            return;
        }
        if (!this._themeBuilder) {
            var themes = (this._options.themes || []).concat(this._options.theme || []);
            var moduleName = this._getModuleEntry(buildData.package);
            this._themeBuilder = new ThemeBuilder_1.ThemeBuilder(themes, this._options.palettes, moduleName);
        }
        var baseSCSSFiles = DefaultConfig_1.DefaultConfig.baseSCSSFiles.concat(DefaultConfig_1.DefaultConfig.baseThemeFiles);
        var scssFiles = buildData.files.scss
            .filter(function (file) { return baseSCSSFiles.indexOf(path.basename(file)) !== -1; });
        var scssBaseContent = CSSBuilder_1.CSSBuilder._loadStyles(scssFiles);
        var themeSCSS = CSSBuilder_1.CSSBuilder._loadStyles(buildData.files.themes);
        var themeCSS = CSSBuilder_1.CSSBuilder._compileSCSS(scssBaseContent + themeSCSS);
        var themeStylesheet = this._themeBuilder.build(themeCSS);
        return CSSBuilder_1.CSSBuilder._buildStylesheet(themeStylesheet, buildData.license);
    };
    /** Generates the license string */
    MaterialBuilder.prototype._getLicense = function (modules) {
        var lines = [
            'AngularJS Material Design',
            'https://github.com/angular/material',
            '@license MIT',
            'v' + this._options.version,
            'Built with: angular-material-tools',
            'Includes modules: ' + modules.join(', '),
            '',
            "Copyright " + new Date().getFullYear() + " Google LLC. All Rights Reserved.",
            'Use of this source code is governed by an MIT-style license that can be ' +
                'found in the LICENSE file at http://material.angularjs.org/LICENSE.'
        ].map(function (line) { return ' * ' + line; });
        lines.unshift('/*!');
        lines.push(' */', '\n');
        return lines.join('\n');
    };
    /** Retrieves the module entry path from the specified package. */
    MaterialBuilder.prototype._getModuleEntry = function (toolsPackage) {
        return path.join(toolsPackage.module, this._options.mainFilename);
    };
    return MaterialBuilder;
}());
exports.MaterialBuilder = MaterialBuilder;
//# sourceMappingURL=MaterialBuilder.js.map