"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThemeBuilder = void 0;
var VirtualContext_1 = require("../virtual_context/VirtualContext");
var DefaultConfig_1 = require("../common/DefaultConfig");
var Utils_1 = require("../common/Utils");
var merge = require('merge');
var ThemeBuilder = /** @class */ (function () {
    /**
     * Instantiates the Theme Builder with the specified themes and possible palettes.
     */
    function ThemeBuilder(theme, palettes, mainModule) {
        // Create a virtual context, to isolate the script which modifies the globals
        // to be able to mock a Browser Environment.
        this._virtualContext = new VirtualContext_1.VirtualContext({
            $$moduleName: mainModule,
            $$interception: {
                run: this._onAngularRunFn.bind(this)
            }
        });
        var injector = this._virtualContext.run(__dirname + '/../resolvers/isolated_angular.js', {
            strictMode: true
        })['injector'];
        this._buildThemingService([].concat(theme), palettes || {}, injector);
    }
    /**
     * Generates a static theme file from the specified theme.
     */
    ThemeBuilder.prototype.build = function (themeCSS) {
        var _this = this;
        var _fakeInjector = {
            // Trim the theme CSS, because the $mdTheming service accidentally introduces a syntax error.
            get: function () { return (themeCSS || _this._$mdThemeCSS).trim(); },
            has: function () { return true; }
        };
        this._generateThemes(_fakeInjector, this._$mdTheming);
        var styleElements = this._virtualContext.globals.document.head.children;
        return styleElements
            .map(function (element) { return element.children[0]['data']; })
            .reduce(function (styleSheet, part) { return styleSheet + part; }, '');
    };
    /**
     * Instantiate the $mdTheming service by calling the provider function with
     * the default color palettes of AngularJS Material.
     *
     * Register the specified theme in the `$mdThemingProvider` by overwriting the default theme.
     * Using the default theme allows users to use the static stylesheet without doing anything.
     *
     * Instantiate the `$mdTheming` service after the provider has been configured.
     */
    ThemeBuilder.prototype._buildThemingService = function (themes, palettes, injector) {
        var _this = this;
        var _colorPalettes = injector['$mdColorPalette'];
        this._$mdThemingProvider = injector['$mdTheming'](_colorPalettes);
        this._$mdThemeCSS = injector['$MD_THEME_CSS'];
        themes.forEach(function (theme) { return _this._registerTheme(theme); });
        Utils_1.Utils.forEach(palettes, function (palette, name) { return _this._registerPalette(name, palette); });
        this._$mdTheming = this._$mdThemingProvider['$get']();
    };
    /** Registers a theme in the $mdTheming provider */
    ThemeBuilder.prototype._registerTheme = function (theme) {
        // Apply defaults theme palettes to current theme.
        theme = merge({}, DefaultConfig_1.DefaultConfig.theme, theme);
        var themeRef = this._$mdThemingProvider
            .theme(theme.name || 'default')
            .primaryPalette(theme.primaryPalette, theme.primaryPaletteHues)
            .accentPalette(theme.accentPalette, theme.accentPaletteHues)
            .warnPalette(theme.warnPalette, theme.warnPaletteHues)
            .backgroundPalette(theme.backgroundPalette, theme.backgroundPaletteHues);
        if (theme.dark) {
            themeRef.dark();
        }
    };
    /** Registers a palette in the $mdTheming provider */
    ThemeBuilder.prototype._registerPalette = function (paletteName, palette) {
        if (palette.extends) {
            var basePalette = palette.extends;
            // Sanitize the palette to fulfill the $mdTheming service requirements
            ThemeBuilder._sanitizePalette(palette);
            palette = this._$mdThemingProvider.extendPalette(basePalette, palette);
        }
        this._$mdThemingProvider.definePalette(paletteName, palette);
    };
    /**
     * Function will be used to intercept AngularJS' Run Phase.
     */
    ThemeBuilder.prototype._onAngularRunFn = function (runFn) {
        if (runFn.name === 'generateAllThemes') {
            this._generateThemes = runFn;
        }
    };
    /**
     * Sanitizes the palette definition from unnecessary overflow, which can lead to errors in the
     * $mdTheming service.
     */
    ThemeBuilder._sanitizePalette = function (palette) {
        delete palette.extends;
    };
    return ThemeBuilder;
}());
exports.ThemeBuilder = ThemeBuilder;
//# sourceMappingURL=ThemeBuilder.js.map