"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerCommands = void 0;
var Utils_1 = require("../common/Utils");
var THEMING_GROUP = 'Theming arguments:';
var PALETTE_OPTIONS = {
    'name': { desc: 'Name for the theme.' },
    'primary-palette': { desc: 'Primary palette color.', alias: 'primary' },
    'accent-palette': { desc: 'Accent palette color.', alias: 'accent' },
    'warn-palette': { desc: 'Warning palette color.', alias: 'warn' },
    'background-palette': { desc: 'Background palette color.', alias: 'background' }
};
// Yargs requires that all commands be marked as strict individually.
var markAsStrict = function (command) { return command.strict(); };
/**
 * Registers all of the available commands.
 */
function registerCommands(yargs) {
    yargs
        .command('', 'Default command. Builds all of the files.', markAsStrict)
        .command('css', 'Builds only the CSS files.', markAsStrict)
        .command('js', 'Builds only the JS files.', markAsStrict)
        .command('theme', 'Builds the theme files for a single theme.', registerThemeArgs('string'), processSingleThemeArgs)
        .command('themes', 'Builds the theme files for multiple themes.', registerThemeArgs('array'), processMultipleThemeArgs);
    return yargs;
}
exports.registerCommands = registerCommands;
/**
 * Registers the theme command arguments.
 */
function registerThemeArgs(argsType) {
    return function (command) {
        Utils_1.Utils.forEach(PALETTE_OPTIONS, function (config, key) {
            command.option(key, {
                alias: config.alias,
                describe: config.desc,
                group: THEMING_GROUP,
                type: argsType,
                requiresArg: true
            });
        });
        command.option('dark', {
            describe: 'Whether to generate a dark theme.',
            group: THEMING_GROUP
        });
        return markAsStrict(command);
    };
}
/**
 * Turns the theme command arguments into a theme object.
 */
function processSingleThemeArgs(args) {
    args.theme = {
        dark: !!args.dark
    };
    Utils_1.Utils.forEach(PALETTE_OPTIONS, function (value, key) {
        if (args[key]) {
            args.theme[Utils_1.Utils.dashToCamel(key)] = args[key];
        }
    });
}
/**
 * Turns the specified theme options into an array of themes. Yargs returns the theme options as:
 * {
 *   primaryPalette: ['red', 'green'],
 *   accentPalette: ['blue', 'yellow']
 * }
 *
 * This method turns them into:
 * [
 *   { primaryPalette: 'red', accentPalette: 'blue' },
 *   { primaryPalette: green, accentPalette: 'yellow' }
 * ]
 */
function processMultipleThemeArgs(args) {
    var themeArgs = Object.keys(PALETTE_OPTIONS).map(function (key) { return Utils_1.Utils.dashToCamel(key); });
    var themes = [];
    // Filter out the defined, theme-specific keys from the rest.
    var definedArgs = Object.keys(args)
        .filter(function (key) { return args[key] && themeArgs.indexOf(key) !== -1; });
    // Find the palette with the most colors to be used as a reference.
    var largestPalette = definedArgs
        .sort(function (key, prevKey) { return args[prevKey].length - args[key].length; })[0];
    if (largestPalette) {
        var _loop_1 = function (i) {
            var newTheme = {
                dark: !!args.dark
            };
            definedArgs.forEach(function (palette) {
                var value = args[palette][i];
                if (value) {
                    newTheme[palette] = value;
                }
            });
            themes.push(newTheme);
        };
        // Goes through the colors and creates new themes by matching them to their index.
        for (var i = 0; i < args[largestPalette].length; i++) {
            _loop_1(i);
        }
    }
    else {
        // Add a default theme, that will use the defaults, if no palettes were specified.
        themes.push({
            dark: !!args.dark
        });
    }
    args.themes = themes;
}
//# sourceMappingURL=commands.js.map