"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PackageResolver = void 0;
var path = require("path");
var fs = require("fs");
var Logger_1 = require("../common/Logger");
var Utils_1 = require("../common/Utils");
var VersionDownloader_1 = require("../common/VersionDownloader");
var NodeModule = require('module');
var execSync = require('child_process').execSync;
// Cross-version compatibility function for pre Node v6 versions.
var getFsConstant = function (name) { return fs.constants ? fs.constants[name] : fs[name]; };
var PackageResolver = /** @class */ (function () {
    function PackageResolver() {
    }
    /**
     * Resolves the directories for a version. The version either gets taken
     * from the cache, or is downloaded via the VersionDownloader.
     * Returns a promise which resolves with the retrieved directories.
     */
    PackageResolver.resolve = function (version, cacheRoot) {
        var _this = this;
        if (version === 'latest') {
            // Fetch the latest version remotely.
            version = this._retrieveLatestVersion();
        }
        else if (version === 'local') {
            // Figure out the version, based on the node_modules.
            version = this._retrieveLocalVersion();
        }
        // Run validators for the current version
        this._validateVersion(version);
        return this._isExisting(path.join(path.resolve(cacheRoot), version))
            .then(this._resolveDirectories)
            .then(function (directories) {
            Logger_1.Logger.info('Using AngularJS Material version from cache.');
            return {
                root: path.join(directories.module, '..'),
                module: directories.module,
                source: directories.source,
                version: version
            };
        })
            .catch(function (directories) {
            directories = _this._resolveDirectories(directories);
            return Promise.all([
                VersionDownloader_1.VersionDownloader.getModuleVersion(version, directories.module),
                VersionDownloader_1.VersionDownloader.getSourceVersion(version, directories.source)
            ]).then(function (downloadPaths) {
                return {
                    root: path.join(directories.module, '..'),
                    module: downloadPaths[0],
                    source: downloadPaths[1],
                    version: version
                };
            });
        });
    };
    /** Creates the directory paths for the cached versions */
    PackageResolver._resolveDirectories = function (cacheDirectory) {
        return {
            module: path.join(cacheDirectory, 'module'),
            source: path.join(cacheDirectory, 'source')
        };
    };
    /** Validates the current resolving version and shows warnings if necessary */
    PackageResolver._validateVersion = function (version) {
        if (Utils_1.Utils.extractVersionNumber(version) < Utils_1.Utils.extractVersionNumber('1.0.0')) {
            Logger_1.Logger.warn('Material-Tools: You are loading an unsupported version. ' +
                'Only >= v1.0.0 versions are fully supported.');
        }
    };
    /**
     * Retrieves the local installed AngularJS Material version form the current PWD.
     */
    PackageResolver._retrieveLocalVersion = function () {
        // Create a Node module which runs at the current process working directory.
        var _cliModule = new NodeModule(process.cwd());
        _cliModule.paths = NodeModule._nodeModulePaths(process.cwd());
        // Resolve the angular-material module form the CWD module.
        var resolvedModule = path.dirname(NodeModule._resolveFilename('angular-material', _cliModule));
        var packageFile = path.join(resolvedModule, 'package.json');
        // Load the version from the local installed AngularJS Material dependency.
        return require(packageFile)['version'];
    };
    /**
     * Checks whether the given path is available in the file system.
     * Returns a promise which resolves / rejects with the given path.
     */
    PackageResolver._isExisting = function (filePath) {
        return new Promise(function (resolve, reject) {
            fs.access(filePath, getFsConstant('R_OK') | getFsConstant('W_OK'), function (doesNotExist) {
                if (doesNotExist) {
                    reject(filePath);
                }
                else {
                    resolve(filePath);
                }
            });
        });
    };
    /**
     * Retrieves the latest AngularJS Material version remotely.
     */
    PackageResolver._retrieveLatestVersion = function () {
        try {
            return execSync('npm view angular-material version --silent').toString().trim();
        }
        catch (e) {
            Logger_1.Logger.error('Failed to retrieve latest version.');
            Logger_1.Logger.info(e.stack);
        }
        return '';
    };
    return PackageResolver;
}());
exports.PackageResolver = PackageResolver;
//# sourceMappingURL=PackageResolver.js.map