"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSandboxRequire = void 0;
var fs = require("fs");
var vm = require("vm");
var path = require("path");
var Utils_1 = require("../common/Utils");
var NodeModule = require('module');
var merge = require('merge');
/** Default Options for the Sandbox Require */
var DEFAULT_OPTIONS = {
    caching: true,
    strictMode: false
};
/**
 * Creates a require function, which runs the required files inside of a new Virtual Machine,
 * which supports the Node Environment inside of the new VM, without accessing the main VM.
 */
function createSandboxRequire(filePath, globals, options, EXPORTS_CACHE) {
    if (EXPORTS_CACHE === void 0) { EXPORTS_CACHE = {}; }
    options = merge({}, DEFAULT_OPTIONS, options || {});
    var _parentModule = new NodeModule(filePath);
    _parentModule.filename = filePath;
    _parentModule.paths = NodeModule._nodeModulePaths(path.dirname(filePath));
    function SandboxRequire(file) {
        var fileName = resolve(file);
        if (EXPORTS_CACHE[fileName] && options.caching) {
            return EXPORTS_CACHE[fileName];
        }
        var fileSource = fs.readFileSync(fileName, 'utf8');
        var currentModule = new NodeModule(fileName, _parentModule);
        currentModule.filename = fileName;
        var childSandboxRequire = createSandboxRequire(fileName, globals, options, EXPORTS_CACHE);
        var locals = getLocals(currentModule, childSandboxRequire);
        fileSource =
            "(function(global, " + Object.keys(locals).join(', ') + ") {\n        " + (options.strictMode ? '"use strict";' : '') + "\n        " + fileSource + "\n      });";
        // Adds the default Node global variables to the custom globals, specified by the developer.
        updateGlobals();
        // Runs the loaded file source inside of a new context with the given globals.
        var runFn = vm.runInNewContext(fileSource, globals, fileName);
        var _localValues = [];
        // Iterate through all locals and push the retrieved values to the value array.
        Utils_1.Utils.forEach(locals, function (value, key) { return _localValues.push(value); });
        // Run our Function inside of the new context.
        runFn.apply(currentModule.exports, [globals].concat(_localValues));
        if (options.caching) {
            EXPORTS_CACHE[fileName] = currentModule.exports;
        }
        return currentModule.exports;
    }
    function updateGlobals() {
        // Moves the default Node globals into the current fake global object.
        Utils_1.Utils.forEach(global, function (value, key) { return globals[key] = value; });
    }
    function getLocals(module, requireFn) {
        return {
            __filename: module.filename,
            __dirname: path.dirname(module.filename),
            module: module,
            exports: module.exports,
            require: requireFn
        };
    }
    function resolve(moduleName) {
        var resolved = NodeModule._resolveFilename(moduleName, _parentModule);
        return (resolved instanceof Array) ? resolved[1] : resolved;
    }
    SandboxRequire['resolve'] = resolve;
    return SandboxRequire;
}
exports.createSandboxRequire = createSandboxRequire;
//# sourceMappingURL=SandboxRequire.js.map